from pyxll import xl_func, plot
import arcticdb as adb
import pandas as pd
import matplotlib.pyplot as plt
import mplfinance as mpf
import datetime as dt



@xl_func
def adb_library(url, library):
    ac = adb.Arctic(url)
    return ac[library]


@xl_func("object, str, datetime, datetime, int: dataframe<index=True>")
def raw_tick_data(lib, symbol, start, end, max_rows=10000):
    data = lib.read(symbol, date_range=[start, end])
    if len(data.data) > max_rows:
        raise RuntimeError("Number of rows is greater than max rows")
    return data.data


@xl_func("object, str, datetime, datetime, str, int: dataframe<index=True>")
def resampled_tick_data(lib, symbol, start, end, freq, max_rows=10000):
    qb = adb.QueryBuilder()
    qb = qb.resample(freq, closed='right').agg({
        'high': ('mid', 'max'),
        'low': ('mid', 'min'),
        'open': ('mid', 'first'),
        'close': ('mid', 'last')
    })

    data = lib.read(symbol,
                    date_range=[start, end],
                    query_builder=qb)

    df = data.data.dropna()

    if max_rows is not None and len(df) > max_rows:
        raise RuntimeError("Number of rows is greater than max rows")
    return df


@xl_func("object, str, datetime, datetime, str, int[], str: var")
def plot_fx_rates(lib, symbol, start, end, freq="1D", mav=[], style="seaborn-v0_8"):

    qb = adb.QueryBuilder()
    qb = qb.resample(freq, closed='right').agg({
        'high': ('mid', 'max'),
        'low': ('mid', 'min'),
        'open': ('mid', 'first'),
        'close': ('mid', 'last')
    })

    data = lib.read(symbol,
                    date_range=[start, end],
                    query_builder=qb)

    df = data.data

    with plt.style.context(style):
        fig, ax = plt.subplots()
        fig.suptitle(symbol)
        ax.grid(True)

        mpf.plot(df,
                 type='candle',
                 mav=[x for x in mav if x],
                 ax=ax)

        fig.tight_layout()
        plot(fig)

    return fig


if __name__ == "__main__":

    import arcticdb as adb
    import polars as pl

    populate_db = True
    csv_glob = "E:/fx-data/DAT_ASCII_EURUSD_T_*.csv"

    # Data downloaded from
    # https://www.histdata.com/download-free-forex-historical-data/?/ascii/tick-data-quotes/eurusd/2023

    ac = adb.Arctic("lmdb://E:/fx-data/arcticdb")
    lib = ac.get_library("fx-test", create_if_missing=True)

    if populate_db:
        # Load the csv data using polars
        df = pl.scan_csv(csv_glob, has_header=False)

        # Extract time, bid, ask and mid
        df = df.select(
            pl.col("column_1").alias("timestamp").str.to_datetime("%Y%m%d %H%M%S%3f", time_unit="ns"),
            pl.col("column_2").alias("bid"),
            pl.col("column_3").alias("ask"),
            (pl.col("column_2") + ((pl.col("column_3") - pl.col("column_2")) / 2)).alias("mid"),
        ).sort("timestamp", descending=False)

        # Convert to pandas and ensure index is sorted
        dfp = df.collect().to_pandas()
        dfp = dfp.set_index("timestamp")
        dfp = dfp.sort_index()

        # Write to arctic
        #lib.delete("EURUSD")
        lib.write("EURUSD", dfp)

    # Test resampling query to 15m intervals
    print("Getting resampled data...")
    df = resampled_tick_data(lib,
                             "EURUSD",
                             dt.date(2023,1,1),
                             dt.date(2023,12,31),
                             "15m",
                             max_rows=None)
    
    print(len(df))
    print("Done!")
